<?php

	$local = dirname(__FILE__) . '/';
	require_once($local . 'adodb/adodb.inc.php');
	require_once($local . 'config.php');

	define("ID_MACRO", "%id");
	
	function bq($inCode) 
	{ 
		return "`$inCode`"; 
	}

	function turboGetConnection($inDb = '')
	{
		$data = turboGetConnectData();
		if ($inDb <> '')
			$data['db'] = $inDb;
		$conn = &ADONewConnection('mysql');	
		$conn->Connect($data['server'], $data['user'], $data['password'], $data['db']);   
    $conn->SetFetchMode(ADODB_FETCH_NUM); 
		return $conn;
	}
	
	function splitSql(&$inSql, $inEol="\n")
	{
		$cmd = '';
		$cmds = Array();
		$lines = explode($inEol, $inSql);
		foreach ($lines as $line)
		{
			$line = trim($line);
			$cmd .= $line;
			$l = strlen($line) - 1;
			if ($l >= 0 && $line[$l] == ';')
			{
				$cmds[] = $cmd;
				$cmd = '';
			}
		}
		return $cmds;		
	}
	
	/*
	function turboGetRecordSetArray(&$inRecords)
	{
		if ($inRecords && method_exists($inRecords, 'GetArray'))
			return $inRecords->GetArray();
		else
			return $inRecords;
	}
	
	function turboGetColumns(&$inRecords)
	{
		$c = $inRecords->FieldCount();
		$result = array();
		for ($i = 0; $i < $c; $i++)
		{
			$column = $inRecords->FetchField($i);
			$result[] = $column;
		}
		return $result;
	}
	
	function turboGetFormattedData(&$inRecords, $inStartFrom = 0, $inLimit = '')
	{
		if (!method_exists($inRecords, 'FieldCount') || $inRecords->FieldCount() == 0)
			return $inRecords;
		//$schema = array();
		//$schema['columnInfo'] = turboGetColumns($inRecords);
		$data = turboGetRecordSetArray($inRecords);
		if ($inLimit)
			$data = array_slice($data, $inStartFrom, $inLimit); 
		return array(
				"data" => $data,
				"schema" => turboGetColumns($inRecords),
				"count" => $inRecords->RecordCount()
		);			
		//$formattedData = array();
		//$formattedData[0] = $data;
		//$formattedData[1] = $schema;
		//$formattedData[2] = $inRecords->RecordCount();
		//return $formattedData;
	}
	
	function turboGetArrayAsData(&$inArray)
	{
		$result = array();
		foreach ($inArray as $row)
			array_push($result, array($row));
		return $result;	
	}
	*/
	
	class turboData
	{
		var $db;
		
		function connect($inDatabase = '')
		{
			if ($this->db) 
				$this->db->Close();
			$this->db = &turboGetConnection($inDatabase);
		}
		
		function get_server_name()
		{
			$data = turboGetConnectData();
			return $data['server'];
		}

		function &check_set(&$inRecordSet)
		{
			if (!$inRecordSet) 
				user_error($this->db->ErrorMsg());
			else		
				return $inRecordSet;
		}
		
		function set_to_array(&$inRecordSet)
		{
			if ($inRecordSet && method_exists($inRecordSet, 'GetArray'))
				return $inRecordSet->GetArray();
			else
				return array();
		}
		
		function list_databases()
		{
			$this->connect();	
			$dbs = $this->db->MetaDatabases();
			if ($this->db->ErrorNo()) 
				user_error($this->db->ErrorMsg());
			return $dbs;
		}
		
		function list_tables($inDatabase = '', $ttype = false, $showSchema = false, $mask=false)
		{
			$this->connect($inDatabase);	
			$tables = $this->db->MetaTables($ttype, $showSchema, $mask);
			if ($this->db->ErrorNo()) 
				user_error($this->db->ErrorMsg());
			return $tables;
		}
		
		function list_all_tables()
		{
			$result = array();
			$dbs = $this->list_databases();
			foreach ($dbs as $db) 
				$result[$db] = $this->list_tables($db);
			return array($this->get_server_name() => $result);
		}
		
		function list_columns($inTable, $inDatabase = '')
		{
			$this->connect($inDatabase);	
			return $this->checkSet($this->db->MetaColumns(bq($inTable)));
		}
		
		function get_row_count($inTable, $inDatabase = '')
		{
			if (!$this->db)
				$this->connect($inDatabase);	
			$set = $this->db->Execute('SELECT COUNT(*) FROM ' . bq($inTable));			
			if ($this->db->ErrorNo()) 
				user_error($this->db->ErrorMsg());
			if ($set && !$set->EOF)
				return $set->fields[0];
			else
				user_error('no count');
		}

		function get_table_schema($inTable, $inDatabase = '', $inOwner = false, $inPrimary = false)
		{
			$this->connect($inDatabase);	
			//$db = &turboGetConnection($inDatabase);	
			$table = bq($inTable); // add bq for tables with spaces!
			return array(
				'columns' => $this->db->MetaColumns($table),
				'keys' => $this->db->MetaPrimaryKeys($table, $inOwner),
				'indexes' => $this->db->MetaIndexes($table, $inPrimary, $inOwner),
				'foreignKeys' => $this->db->MetaForeignKeys($table, $inOwner),
				'count' => $this->get_row_count($inTable)
			);				
		}
		
		function _execute_sql($inQuery)
		{
			$set = $this->db->Execute($inQuery);
			if ($this->db->ErrorNo()) 
				user_error($this->db->ErrorMsg());
			return $this->set_to_array($set);
		}
		
		function execute_sql($inQuery, $inDatabase = '')
		{
			$this->connect($inDatabase);	
			return $this->_execute_sql($inQuery);
		}

		function _execute_sql_queue($inSql)
		{
			//$cmds = &splitSql($inSql);
			$cmds = $inSql;
			$results = array();
			foreach ($cmds as $cmd) 
			{
				$set = &$this->db->Execute($cmd);
				if ($this->db->ErrorNo()) 
					user_error($this->db->ErrorMsg());
				else
					$results[] = $set;	
			}		
			return $results;
		}		
		
		function execute_sql_queue($inSql, $inDatabase = '')
		{
			$this->connect($inDatabase);	
			return $this->_execute_sql_queue($inSql);
		}		
		
		function update_reflect($inUpdateQuery, $inFetchQuery, $inDatabase = '')
		{
			$this->connect($inDatabase);	
			$editOk = $this->db->Execute($inUpdateQuery);
			if (!$editOk)
				user_error($this->db->ErrorMsg());
			$affectedRows = $this->db->Affected_Rows();	
			// process auto increment.
			if (strpos($inFetchQuery, ID_MACRO) !== false)
			{
				$insert_id = $this->db->Insert_ID();
				if (!$insert_id)
					user_error($this->db->ErrorMsg());	
				$inFetchQuery = str_replace(ID_MACRO, $insert_id, $inFetchQuery);
			}
			$result = $this->_execute_sql($inFetchQuery);			
			// affected rows added as first return element of array
			array_push($result, $affectedRows);
			return $result;
		}

		// returns false if fetchSql returned any results
		function delete_row($inDeleteSql, $inFetchSql, $inDatabase = '')
		{
			$this->connect($inDatabase);	
			$this->_execute_sql($inDeleteSql);
			$rs = $this->_execute_sql($inFetchSql);			
			return (!$rs || $rs->RecordCount() == 0);
			/*
			$this->execute_sql_queue($inDeleteSql, $inDatabase);
			$results = $this->execute_sql_queue($inFetchQuery, $inDatabase);			
			foreach ($results as $rs)
				if ($rs->RecordCount() > 0)
					return false;
			return true;
			*/
		}
		
		function delete_queue($inQueue, $inDatabase = '')
		{
			$this->connect($inDatabase);	
			$deleted = 0;
			$error = '';
			foreach ($inQueue as $cmd) 
			{
				$set = &$this->db->Execute($cmd);
				if ($this->db->ErrorNo()) 
				{
					$error = $this->db->ErrorMsg();
					break;
				}
				else
					$deleted++;
			}		
			return Array("deleted" => $deleted, "error" => $error);
		}		
		
		// returns false if fetchSql returned any results
		function delete_rows($inDeleteSql, $inFetchSql, $inDatabase = '')
		{
			$this->connect($inDatabase);	
			$this->_execute_sql_queue($inDeleteSql);
			$results = $this->_execute_sql_queue($inFetchSql);			
			foreach ($results as $rs)
				if ($rs->RecordCount() > 0)
					return false;
			return true;
		}
		
		/*		
		function select_limit($inQuery, $inDatabase = '', $inOffset, $inLimit)
		{
			$db = &turboGetConnection($inDatabase);	
			return $this->set_to_array($db->SelectLimit($inQuery, $inLimit, $inOffset));
			//if (!$recordSet) 
			//	user_error($db->ErrorMsg());
			//else	
			//	return turboGetRecordSetArray($recordSet);
		}

		function execute_sql_queue($inSql, $inDatabase = '')
		{
			$db = &turboGetConnection($inDatabase);	
			$cmds = &splitSql($inSql);
			$results = array();
			foreach ($cmds as $cmd) 
			{
				$rs = &$db->Execute($cmd);
				if (!$rs) 
					user_error($db->ErrorMsg());
				else
					$results[] = $rs;	
			}		
			return $results;
		}
		
		function execute_sql_formatted($inQuery, $inDatabase = '', $inStartFrom = 0, $inLimit = '')
		{
			$db = &turboGetConnection($inDatabase);	
			$recordSet = &$db->Execute($inQuery);
			if (!$recordSet) 
				user_error($db->ErrorMsg());
			else	
				return turboGetFormattedData($recordSet, $inStartFrom, $inLimit);
		}
		
		function rename_table($inOldName, $inNewName, $inDatabase = '')
		{
			$db = &turboGetConnection($inDatabase);	
			$dict = NewDataDictionary($db);
			$sqlarray = $dict->RenameTableSQL($inOldName, $inNewName);
			if (!$dict->ExecuteSQLArray($sqlarray))
				user_error($db->ErrorMsg());
		}
		
		function rename_tables($inTables, $inNewTables, $inDatabase = '')
		{
			for ($i = 0; $i < count($inTables); $i++)
			{
				if ($inNewTables[$i] && $inTables[$i] != $inNewTables[$i])
					$this->rename_table($inTables[$i], $inNewTables[$i], $inDatabase);
			}	
		}
		
		// returns true if none of the deleted rows can be fetched, or false if they can be (problem with delete)
		function delete_records($inDeleteSql, $inFetchSql, $inDatabase = '')
		{
			$db = &turboGetConnection($inDatabase);	
			//$deleteOk = $this->executeSqlQueue($inDeleteSql);
			$this->execute_sql_queue($inDeleteSql, $inDatabase);
			//$editOk = &$db->Execute($inEditQuery);
			//if (!$deleteOk)
			//	user_error($db->ErrorMsg());
			$results = $this->execute_sql_queue($inFetchQuery, $inDatabase);			
			foreach ($results as $rs)
				if ($rs->RecordCount() > 0)
					return false;
			return true;
		}
		
		function get_rows($inTable, $inLimit = 0)
		{
			$sql = "SELECT * FROM ";
			$sql .= ($inTable ? bq($inTable) : '');
			$sql .= ($inLimit ? " LIMIT $inLimit" : '');
			return $this->execute_sql($sql);
		}
		
		function update_row($inTable, $inData, $inWhere)
		{
			$db = &turboGetConnection();	
			$recordSet = $db->AutoExecute($inTable, $inData, 'UPDATE', $inWhere);
			if (!$recordSet) 
				user_error($db->ErrorMsg());
			return true;
		}
		
		function get_primary_keys($inTable, $inDatabase = '', $inOwner = false)
		{
			$db = &turboGetConnection($inDatabase);	
			$primaryKeys = $db->MetaPrimaryKeys(bq($inTable), $inOwner);
			if (!$primaryKeys) 
				user_error($db->ErrorMsg());
			return $primaryKeys;
		}
		
		function get_column_info($inTable, $inDatabase = '')
		{
			$this->connect($inDatabase);	
			$recordSet = $db->MetaColumns(bq($inTable));
			if (!$recordSet) 
				user_error($db->ErrorMsg());
			else 	
				return $recordSet;
		}
		*/
	}
	
?>